<?php
// -----------------------------------------
// Das Script kann unter Verwendung dieses Vermerks uneingeschrnkt
// genutzt / verndert werden.
//  www.php-space.info
// -----------------------------------------

/**
 * Klasse fr die Sprachverwaltung.
 *
 * Hier werden alle auszugebenden Texte (Meldungen, Fehlertexte usw.)
 * zentral verwaltet. Bei der Instanziierung wird die gewnschte Sprache
 * (z. B. 'de' oder 'en') angegeben.
 */
class Sprache
{
  /** @var string Die eingestellte Sprache (z. B. 'de' oder 'en') */
  private string $language;

  /** @var array Die bersetzungen */
  private array $messages;

  /**
   * Konstruktor der Klasse Sprache.
   *
   * Dieser Konstruktor initialisiert die Sprachverwaltung, indem er die gewnschte Sprache
   * festlegt und die entsprechenden bersetzungen ldt. Untersttzt werden die Sprachen
   * Deutsch ('de'), Englisch ('en'), Franzsisch ('fr'), Spanisch ('es') und Italienisch ('it').
   * Falls eine nicht untersttzte Sprache angegeben wird, wird standardmig Deutsch ('de') verwendet.
   *
   * Ablauf:
   * 1. berprfung, ob der bergebene Sprachcode zu den untersttzten Sprachen gehrt.
   * 2. Falls der Sprachcode nicht untersttzt wird, wird er auf 'de' gesetzt.
   * 3. Initialisierung der Sprachvariable mit dem gltigen Sprachcode.
   * 4. Definition der bersetzungen fr die untersttzten Sprachen und Zuweisung zu der
   *    Instanzvariablen $messages.
   *
   * @param string $language Der Sprachcode, der die gewnschte Sprache angibt. Untersttzt werden:
   *                         'de' (Deutsch), 'en' (Englisch), 'fr' (Franzsisch), 'es' (Spanisch), 'it' (Italienisch).
   *
   * Beispiele fr den Aufruf:
   * <code>
   * $spracheDeutsch = new Sprache('de'); // Initialisiert die Sprachverwaltung auf Deutsch
   * $spracheEnglisch = new Sprache('en'); // Initialisiert die Sprachverwaltung auf Englisch
   * $spracheFranzoesisch = new Sprache('fr'); // Initialisiert die Sprachverwaltung auf Franzsisch
   * $spracheUngueltig = new Sprache('jp'); // Initialisiert die Sprachverwaltung auf Deutsch, da 'jp' nicht untersttzt wird
   * </code>
   */
  public function __construct(string $language)
  {
    // Erlaubte Sprachen: 'de', 'en', 'fr', 'es' und 'it' - alles andere wird zu 'de'
    if (!in_array($language, ['de', 'en', 'fr', 'es', 'it'])) {
      $language = 'de';
    }
    $this->language = $language;

    // Definition der bersetzungen fr die untersttzten Sprachen
    $this->messages = [
      'de' => [
        'visitorMessage'      => 'Sie sind Besucher Nr. %d',
        'visitorMessageJS'    => "Sie sind Besucher Nr. %s",
        'genericError'        => 'Ein Fehler ist aufgetreten. Bitte versuchen Sie es spter erneut.',
        'errorFileNotExists'  => 'Datei existiert nicht: %s',
        'errorReadFile'       => 'Fehler beim Lesen der Datei: %s',
        'errorWriteFile'      => 'Fehler beim Schreiben in die Datei: %s',
      ],
      'en' => [
        'visitorMessage'      => 'You are visitor no. %d',
        'visitorMessageJS'    => "You are visitor no. %s",
        'genericError'        => 'An error occurred. Please try again later.',
        'errorFileNotExists'  => 'File does not exist: %s',
        'errorReadFile'       => 'Error reading file: %s',
        'errorWriteFile'      => 'Error writing to file: %s',
      ],
      'fr' => [
        'visitorMessage'      => 'Vous tes le visiteur n %d',
        'visitorMessageJS'    => "Vous tes le visiteur n %s",
        'genericError'        => "Une erreur s'est produite. Veuillez ressayer plus tard.",
        'errorFileNotExists'  => 'Le fichier n\'existe pas: %s',
        'errorReadFile'       => 'Erreur lors de la lecture du fichier: %s',
        'errorWriteFile'      => 'Erreur lors de l\'criture dans le fichier: %s',
      ],
      'es' => [
        'visitorMessage'      => 'Eres el visitante n %d',
        'visitorMessageJS'    => "Eres el visitante n %s",
        'genericError'        => 'Ocurri un error. Por favor, intntalo de nuevo ms tarde.',
        'errorFileNotExists'  => 'El archivo no existe: %s',
        'errorReadFile'       => 'Error al leer el archivo: %s',
        'errorWriteFile'      => 'Error al escribir en el archivo: %s',
      ],
      'it' => [
        'visitorMessage'      => 'Sei il visitatore n. %d',
        'visitorMessageJS'    => "Sei il visitatore n. %s",
        'genericError'        => 'Si  verificato un errore. Riprova pi tardi.',
        'errorFileNotExists'  => 'Il file non esiste: %s',
        'errorReadFile'       => 'Errore durante la lettura del file: %s',
        'errorWriteFile'      => 'Errore durante la scrittura nel file: %s',
      ],
    ];
  }

  /**
   * Liefert den bersetzten Text fr den angegebenen Schlssel.
   *
   * @param string $key Der Schlssel der Meldung.
   * @return string Der bersetzte Text oder ein leerer String, falls nicht gefunden.
   */
  public function getTranslation(string $key): string
  {
    return $this->messages[$this->language][$key] ?? '';
  }
}

/**
 * Klasse zur Verwaltung eines Besucherzhlers.
 */
class BesucherZaehler
{
  /** @var string Pfad zur Datei, in der der Zhler gespeichert wird */
  private string $dateipfad;

  /** @var Sprache Instanz der Sprachklasse zur Ausgabe von Meldungen */
  private Sprache $sprache;

  /**
   * Konstruktor der Klasse.
   *
   * @param string  $dateipfad Der Pfad zur Datei, die den Besucherzhler enthlt.
   * @param Sprache $sprache   Instanz der Sprachklasse zur Ausgabe von Meldungen.
   */
  public function __construct(string $dateipfad, Sprache $sprache)
  {
    $this->dateipfad = $dateipfad;
    $this->sprache   = $sprache;
  }

  /**
   * Zhlt die Anzahl der Besucher, indem eine Datei gelesen und aktualisiert wird.
   *
   * @return int|false Die aktuelle Besucherzahl oder false im Fehlerfall.
   */
  public function zaehleBesucher(): int|false
  {
    try {
      if (!file_exists($this->dateipfad)) {
        throw new RuntimeException(sprintf(
          $this->sprache->getTranslation('errorFileNotExists'),
          $this->dateipfad
        ));
      }

      $content = file_get_contents($this->dateipfad);
      if ($content === false) {
        throw new RuntimeException(sprintf(
          $this->sprache->getTranslation('errorReadFile'),
          $this->dateipfad
        ));
      }

      $iZaehler = (int)$content;
      $iZaehler++;

      if (file_put_contents($this->dateipfad, (string)$iZaehler) === false) {
        throw new RuntimeException(sprintf(
          $this->sprache->getTranslation('errorWriteFile'),
          $this->dateipfad
        ));
      }

      return $iZaehler;
    } catch (RuntimeException $e) {
      // Fehlerbehandlung: Loggen oder weitere Aktionen durchfhren
      error_log($e->getMessage());
      return false;
    }
  }

  /**
   * Holt eine URL-Variable und filtert sie.
   *
   * @param string $variablenname Der Name der URL-Variable.
   * @return string Der gefilterte Wert der URL-Variable oder ein leerer String, wenn sie nicht gesetzt ist.
   */
  public function getUrlvar(string $variablenname): string
  {
    return isset($_GET[$variablenname])
      ? htmlspecialchars(trim($_GET[$variablenname]))
      : "";
  }

  /**
   * Gibt die Besucherzahl aus.
   *
   * Dabei wird anhand eines GET-Parameters ("ac") geprft, ob die Ausgabe
   * im JavaScript-Format erfolgen soll. Der eigentliche Text wird ber die
   * Sprachklasse bezogen.
   *
   * @param int $iZaehler Die aktuelle Besucherzahl.
   */
  public function ausgabeBesucher(int $iZaehler): void
  {
    $sAc = $this->getUrlvar("ac");
    if ($sAc === 'js') {
      echo 'document.write(\''.sprintf(
        $this->sprache->getTranslation('visitorMessageJS'),
        $iZaehler
      ).'\');';
    } else {
      echo sprintf(
        $this->sprache->getTranslation('visitorMessage'),
        $iZaehler
      );
    }
  }
}

// -------------------------
// Nutzung des Scripts
// -------------------------

// Sprache festlegen: hier z. B. 'de' fr Deutsch, 'en' fr Englisch.
// Achtung: Die Sprache wird NICHT ber GET-Parameter bestimmt, sondern direkt hier eingestellt.
$languageCode = 'de';  // Alternativ: 'en', 'it', 'fr', 'es'
$sprache = new Sprache($languageCode);

// Besucherzhler initialisieren (Dateipfad ggf. anpassen)
$besucherZaehler = new BesucherZaehler('./daten/cont.txt', $sprache);
$iZaehler = $besucherZaehler->zaehleBesucher();

// Ausgabe der Besucherzahl oder Fehlermeldung (sprachabhngig)
if ($iZaehler !== false) {
  $besucherZaehler->ausgabeBesucher($iZaehler);
} else {
  echo $sprache->getTranslation('genericError');
}